<?php

require_once 'base/FpagfipmagnetEntityBase.php';

/**
 * FpagfipmagnetEntity
 *
 * @package Entity
 * @author  alessandrobartels
 */
class FpagfipmagnetEntity extends FpaGfipmagnetEntityBase
{


    /**
     * Retorna registros.
     *
     * @param TkDateTime $mesCompetencia Mes de competencia.
     * @param integer    $nrOrg          Numero da organizacao.
     *
     * @return array FpagfipmagnetEntity
     *
     * @author alessandrobartels - 03/06/2011
     */
    public function retornaRegistros(TkDateTime $mesCompetencia, $nrOrg){
        return Fpagfipmagnet::retornaRegistros($mesCompetencia, $nrOrg);
    }


    /**
     * Cria lista de objetos FpagfipmagnetEntity conforme array de tipo de registros.
     * Nao usado para tipo de registro com informa��es do trabalhador (30 e 32).
     *
     * @param string $mesCompetencia    Mes de competencia.
     * @param string $listaTipoRegistro Lista com os tipos de registros.
     * @param string $nrOrg             Numero da Organizacao.
     *
     * @return boolean
     *
     * @author alessandrobartels - 02/06/2011
     */
    public function inicializaListaRegistros($mesCompetencia, $listaTipoRegistro, $nrOrg){
        $resultados = Fpagfipmagnet::retornaRegistrosPorTipo($mesCompetencia, $listaTipoRegistro, $nrOrg);
        if (is_array($resultados) === true) {
            foreach ($resultados as $resultado) {
                $this->addItem($resultado, $resultado->getNrgfipmagnet());
            }

            return true;
        } else {
            return false;
        }
    }


    /**
     * Adiciona registro na lista.
     * Nao usado para tipo de registro com informa��es do trabalhador (30 e 32).
     *
     * @param string $nrOrg                Numero da Organizacao.
     * @param string $mesCompetencia       Mes de competencia.
     * @param string $tipoRegistro         Numero do tipo de registro.
     * @param string $tipoInscricao        Numero do tipo de inscricao.
     * @param string $nrInscricao          Numero da inscricao.
     * @param string $linha                Linha do registro GFIP.
     * @param string $tipoInscricaoTom     Tipo de inscricao do tomador de servico.
     * @param string $nrInscricaoTom       Numero de inscricao do tomador de servico.
     * @param string $nrVinculo            Numero do vinculo.
     * @param string $dtAdmissao           Data de Admissao.
     * @param string $nrPisPasepCi         Numero de inscricao PIS / PASEP / CI.
     * @param string $vrSalarioFamilia     Valor do Salario Familia.
     * @param string $vrSalarioMaternidade Valor do Salario Maternidade.
     * @param string $categoriaTrabalhador Categoria do trabalhador.
     *
     * @return void
     *
     * @author alessandrobartels - 02/06/2011
     */
    public function insereRegistroLista(
        $nrOrg, $mesCompetencia, $tipoRegistro, $tipoInscricao, $nrInscricao, $linha,
        $tipoInscricaoTom = "0", $nrInscricaoTom = "0", $nrVinculo = null, $dtAdmissao = null, $nrPisPasepCi = "0",
        $vrSalarioFamilia = null, $vrSalarioMaternidade = null, $categoriaTrabalhador = null, $nmrazaoestrut = null
    ){

        $item = new self();

        $item->setIdativo('S'); // Verificar.
        $item->setNrorg($nrOrg);
        $item->setMescompetencia($mesCompetencia);
        $item->setIdtiporegistro($tipoRegistro);
        $item->setIdtpinscestrutu($tipoInscricao);
        $item->setCdinscestrut($nrInscricao);
        $item->setDslinharegistro($linha);

        $item->setIdtpinscesttoma($tipoInscricaoTom);
        $item->setCdinscestrutoma($nrInscricaoTom);
        $item->setNrvinculom($nrVinculo);
        $item->setDtadmissaovinc($dtAdmissao);
        $item->setNrpispaseppes($nrPisPasepCi);
        $item->setVrsalfamilia($vrSalarioFamilia);
        $item->setVrsalmaternidad($vrSalarioMaternidade);
        $item->setIdcategtrab($categoriaTrabalhador);
        $item->setNmrazsocestrut($nmrazaoestrut);

        $this->addItem($item);

    }


    /**
     * Insere ou atualiza registro.
     *
     * @param string $nrOrg                Numero da organizacao.
     * @param string $mesCompetencia       Mes de competencia.
     * @param string $tipoRegistro         Tipo de registro SEFIP.
     * @param string $tipoInscricao        Tipo de inscricao.
     * @param string $nrInscricao          Numero de inscricao.
     * @param string $tipoInscricaoTomador Tipo de inscricao do tomador.
     * @param string $nrInscricaoTomador   Numero de inscricao do tomador.
     * @param string $linha                Linha do registro.
     * @param string $nrVinculoM           Numero do vinculo.
     * @param string $dataAdmissao         Data de admiss�o do vinculo.
     * @param string $nrPisPasepCi         Numero de inscricao (PIS/PASEP/CI).
     * @param string $vrSalarioFamilia     Valor do salario familia.
     * @param string $vrSalarioMaternidade Valor do salario maternidade.
     * @param string $categoriaTrabalhador Categoria do trabalhador.
     *
     * @return void
     */
    public function persisteRegistro(
        $nrOrg, $mesCompetencia, $tipoRegistro, $tipoInscricao, $nrInscricao, $tipoInscricaoTomador,
        $nrInscricaoTomador, $linha, $nrVinculoM, $dataAdmissao, $nrPisPasepCi, $vrSalarioFamilia,
        $vrSalarioMaternidade, $categoriaTrabalhador, $nmrazaoestrut = null
    ){
        // Procura o registro na base.
        $item = Fpagfipmagnet::retornaRegistrosTrabalhador(
            $mesCompetencia, $nrOrg, $tipoRegistro, $tipoInscricao, $nrInscricao, $tipoInscricaoTomador, $nrInscricaoTomador,
            $nrPisPasepCi, $dataAdmissao, $categoriaTrabalhador
        );

        if (is_object($item) === true) {
            $item->setNrgfipmagnet($item->getNrgfipmagnet());
            $item->setVrsalfamilia($vrSalarioFamilia);
            $item->setVrsalmaternidad($vrSalarioMaternidade);
            $item->setDslinharegistro($linha);
            //$item->getDtultatu(now());
        } else {
            $item = new self();

            $item->setIdativo('S'); // Verificar.
            $item->setNrorg($nrOrg);
            $item->setMescompetencia($mesCompetencia);
            $item->setIdtiporegistro($tipoRegistro);
            $item->setIdtpinscestrutu($tipoInscricao);
            $item->setCdinscestrut($nrInscricao);
            $item->setIdtpinscesttoma($tipoInscricaoTomador);
            $item->setCdinscestrutoma($nrInscricaoTomador);
            $item->setNrvinculom($nrVinculoM);
            $item->setDtadmissaovinc($dataAdmissao);
            $item->setNrpispaseppes($nrPisPasepCi);
            $item->setVrsalfamilia($vrSalarioFamilia);
            $item->setVrsalmaternidad($vrSalarioMaternidade);
            $item->setIdcategtrab($categoriaTrabalhador);
            $item->setDslinharegistro($linha);
            $item->setNmrazsocestrut($nmrazaoestrut);
        }

        $item->persist();
    }


    /**
     * Procura registro na lista.
     *
     * @param string $nrInscricao    Numero da inscricao da empresa.
     * @param string $nrInscricaoTom Numero da inscricao do tomador.
     * @param string $nrRegistro     Numero do tipo do registro.
     *
     * @return boolean
     *
     * @author alessandrobartels - 02/06/2011
     */
    public function procuraRegistroLista($nrInscricao, $nrInscricaoTom, $nrRegistro, $nmRazsocestrut = null) {
        $itens = $this->getItems();
        foreach ($itens as $key => $item) {
            if (($item->getCdinscestrut() === $nrInscricao)
                && ($item->getCdinscestrutoma() === $nrInscricaoTom)
                && ($item->getIdtiporegistro() === $nrRegistro)
            ) {
                return $key;
            }
        }    

//        if ($nmRazsocestrut != null)  {
//            foreach ($itens as $key => $item) {                                
//                if (($item->getCdinscestrut() === $nrInscricao)
//                   && ($item->getCdinscestrutoma() === $nrInscricaoTom)
//                    && ($item->getIdtiporegistro() === $nrRegistro)
//                    && (strpos($item->getDslinharegistro(), $nmRazsocestrut) !== false)
//                ) {
//                    return $key;
//                }
//            }
//        } else {
//            foreach ($itens as $key => $item) {
//            if (($item->getCdinscestrut() === $nrInscricao)
//                && ($item->getCdinscestrutoma() === $nrInscricaoTom)
//                && ($item->getIdtiporegistro() === $nrRegistro)
//            ) {
//                return $key;
//            }
//        }
 //       }

        return false;
    }

    /**
     * Remove todos os registros da Org. no mes de competencia.
     *
     * @param integer $nrOrg          Numero da Organizacao.
     * @param string  $mesCompetencia Mes de competencia.
     *
     * @return void
     *
     * @author alessandrobartels - 04/01/2012
     */
    public function retroceder($nrOrg, $mesCompetencia){

        $registros = $this->retornaRegistros($mesCompetencia, $nrOrg);
        if (count($registros) > 0) {
            foreach ($registros as $registro) {
                $this->addItem($registro);
                $this->removeItem($registro);
            }

            $this->persist();
        }
    }


}