/*
 * This class acumulate every generical events in zeedhi
 * Based on https://github.com/JeremyLikness/jsEventAgg/blob/master/jsEventAgg.jss
 */
var EventAggregator = (function() {

	function EventAggregator(eventEngine) {
		this._messages = {};
		var self = this;

		var events = [
			"EventOnRequestMetaDataStart",
			"EventOnRequestStart",
			"EventOnRequestEnd",
			"EventOnRequestRetry",
			"EventOnRequestSuccess",
			"EventOnRequestError",
			"EventOnRequestIdle"
		];

		var CONFIGURABLE_REQUEST_EVENTS = {
			"EventOnRequestMetaDataStart": true,
			"EventOnRequestStart": true
		};

		var eventHandlerFactory = function eventHandlerFactory(eventName) {
			return function(event, currentRequest) {
				var eventData = {
					e: event,
					currentRequest: currentRequest
				};
				if (isConfigurableRequestEvent(eventName)) {
					eventData.requestConfig = event && event.e? event.e.requestConfig : null;
				}
				self.publish(eventName, eventData);
			};
		};

		function isConfigurableRequestEvent(eventName) {
			return !!CONFIGURABLE_REQUEST_EVENTS[eventName];
		}

		var eventToBind = events.map(function(eventName) {
			return {
				name: eventName,
				code: eventHandlerFactory(eventName)
			};
		});
		eventEngine.bindEvents(self, eventToBind);
		this.onRequestSuccess(function(e, currentRequest) {
			self.onRequestEnd(e, currentRequest);
		});
		this.onRequestRetry(function(e, currentRequest) {
			self.onRequestEnd(e, currentRequest);
		});
		this.onRequestError(function(e, currentRequest) {
			self.onRequestEnd(e, currentRequest);
		});
	}

	EventAggregator.prototype.subscribe = function(message, callback) {
		var msg;
		msg = this._messages[message] || (this._messages[message] = new Message());
		return msg.subscribe(callback);
	};

	EventAggregator.prototype.unSubscribe = function(message, token) {
		if (this._messages[message]) {
			((this._messages[message])).unSubscribe(token);
		}
	};

	EventAggregator.prototype.publish = function(message, payload) {
		if (this._messages[message]) {
			((this._messages[message])).notify(payload);
		}
	};

	function Subscription(id, callback) {
		this.id = id;
		this.callback = callback;
	}

	var Message = (function() {
		function Message() {
			this._subscriptions = [];
			this._nextId = 0;
		}
		Message.prototype.subscribe = function(callback) {
			var subscription = new Subscription(this._nextId++, callback);
			this._subscriptions[subscription.id] = subscription;
			return subscription.id;
		};
		Message.prototype.unSubscribe = function(id) {
			delete this._subscriptions[id];
		};
		Message.prototype.notify = function(payload) {
			var index;
			for (index = 0; index < this._subscriptions.length; index++) {
				if (this._subscriptions[index]) {
					this._subscriptions[index].callback(payload);
				}
			}
		};
		return Message;
	})();

	EventAggregator.ON_CONTAINER_OPEN = 'onContainerOpen';

	return EventAggregator;
})();