describe('ZhLayoutCreateLayoutOptionsController', function(){

    var messageService;
    var zhLayoutPreferences;
    var screenService;
    var zhPromise;
    var zhLayoutWidgetsBuilder;
    var zhLayoutCreateLayoutFieldOptionsController;

    var zhLayoutCreateLayoutOptionsController;

    beforeEach(function(){
        messageService          = jasmine.createSpyObj('MessageService',         ['showMessageByCode']);
        zhLayoutPreferences     = jasmine.createSpyObj('ZhLayoutPreferences',    ['saveLayouts', 'applyLayout', 'updateLayouts']);
        screenService           = jasmine.createSpyObj('ScreenService',          ['i18n', 'openPopup', 'closePopup']);
        zhLayoutWidgetsBuilder  = jasmine.createSpyObj('ZhLayoutWidgetsBuilder', ['factoryBuildLayoutOptionsWidget']);
        metaDataFactory         = jasmine.createSpyObj('metaDataFactory',        ['fieldFactory']);
        zhPromise = Q;

        zhLayoutCreateLayoutOptionsController = new ZhLayoutCreateLayoutOptionsController(messageService, zhLayoutPreferences, screenService, zhLayoutWidgetsBuilder, metaDataFactory, zhPromise);
    });

    it('getCreateLayoutOptionsWidget', function(done){
        var preference = {
            'ID': 1,
            'PDF_REPORT_ORIENTATION': 'LANDSCAPE',
            'LAYOUT_CHANGES': '{"widgets": [{"id": "name", "widgets": [{"id": "listWidget", "fields": [{"id": 1, "isVisible": true},{"id": 2,"isVisible": true,"sequenceInList": 0}]}]}]}'
        };

        var listWidget = {
            'id': 'listWidget',
            'label': 'Report name',
            'dataSource': {
                'data': []
            },
            'fields': [
                {
                    'id': 1,
                    'isVisible': true,
                },
                {
                    'id': 2,
                    'isVisible': true,
                    'sequenceInList': 0
                }
            ]
        };

        var configWidget = {
            'id': 'layoutConfigWidget',
            'listWidget': listWidget
        };

        var layoutOptionsWidget = {
            'id': 'layoutOptionsWidget',
            'dataSource': {'name': 'dataSource'},
            'configWidget': configWidget
        };

        screenService.i18n.and.callFake(function(word){
            return word;
        });

        Util.extend = jasmine.createSpy('extend').and.callFake(function(dest, source){
            return source;
        });

        metaDataFactory.fieldFactory.and.callFake(function(field, parentWidget){
            return field;
        });

        zhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget.and.callFake(function(){
            return Q.when(layoutOptionsWidget);
        });

        zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(factoredLayoutOptionsWidget){
            expect(factoredLayoutOptionsWidget).toBe(layoutOptionsWidget);

            zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(factoredLayoutOptionsWidgetSecondCall){
                expect(screenService.i18n).toHaveBeenCalledWith('Layout');
                expect(factoredLayoutOptionsWidgetSecondCall).toBe(factoredLayoutOptionsWidget);
                expect(zhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget.calls.count()).toEqual(1);
                expect(layoutOptionsWidget.fields).toEqual([{'id': 1, 'isVisible': true}, {'id': 2, 'isVisible': true, sequenceInList: 0}]);
                done();
            });
        });
    });

    it('getCreateLayoutOptionsWidget: no change on layout', function(done){
        var preference = {
            'ID': 1,
            'LABEL_TO_PREFERENCE': 'Layout Name',
            'LAYOUT_CHANGES': '{"widgets": [{"id": "name", "widgets": [{"id": "listWidget", "fields": []}]}]}'
        };

        var listWidget = {
            'id': 'listWidget',
            'dataSource': {
                'data': []
            },
            'fields': [
                {
                    'id': 1,
                    'name': 'field 1',
                    'label': 'Field 1',
                    'isVisible': true,
                },
                {
                    'id': 2,
                    'name': 'field 2',
                    'label': 'Field 2',
                    'isVisible': true,
                    'sequenceInList': 0
                }
            ]
        };

        var configWidget = {
            'id': 'layoutConfigWidget',
            'listWidget': listWidget
        };

        var layoutOptionsWidget = {
            'id': 'layoutOptionsWidget',
            'dataSource': {'name': 'dataSource'},
            'configWidget': configWidget
        };

        screenService.i18n.and.callFake(function(word){
            return word;
        });

        metaDataFactory.fieldFactory.and.callFake(function(field, parentWidget){
            return field;
        });

        Util.extend = jasmine.createSpy('extend').and.callFake(function(dest, source){
            return source;
        });

        zhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget.and.callFake(function(){
            return Q.when(layoutOptionsWidget);
        });

        zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(factoredLayoutOptionsWidget){
            expect(factoredLayoutOptionsWidget).toBe(layoutOptionsWidget);

            zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(factoredLayoutOptionsWidgetSecondCall){
                expect(screenService.i18n).toHaveBeenCalledWith('Layout');
                expect(factoredLayoutOptionsWidgetSecondCall).toBe(factoredLayoutOptionsWidget);
                expect(zhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget.calls.count()).toEqual(1);
                expect(layoutOptionsWidget.fields).toEqual(listWidget.fields);
                done();
            });
        });
    });

    it('getCreateLayoutOptionsWidget: no change on field', function(done){
        var preference = {
            'ID': 1,
            'LABEL_TO_PREFERENCE': 'Layout Name',
            'LAYOUT_CHANGES': '{"widgets": [{"id": "name", "widgets": [{"id": "listWidget", "fields": [{"id": 1}]}]}]}'
        };

        var listWidget = {
            'id': 'listWidget',
            'dataSource': {
                'data': []
            },
            'fields': [
                {
                    'id': 1,
                    'name': 'field 1',
                    'label': 'Field 1',
                    'isVisible': true,
                },
                {
                    'id': 2,
                    'name': 'field 2',
                    'label': 'Field 2',
                    'isVisible': true,
                    'sequenceInList': 0
                }
            ]
        };

        var configWidget = {
            'id': 'layoutConfigWidget',
            'listWidget': listWidget
        };

        var layoutOptionsWidget = {
            'id': 'layoutOptionsWidget',
            'dataSource': {'name': 'dataSource'},
            'configWidget': configWidget
        };

        screenService.i18n.and.callFake(function(word){
            return word;
        });

        metaDataFactory.fieldFactory.and.callFake(function(field, parentWidget){
            return field;
        });

        Util.extend = jasmine.createSpy('extend').and.callFake(function(dest, source){
            return source;
        });

        zhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget.and.callFake(function(){
            return Q.when(layoutOptionsWidget);
        });

        zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(factoredLayoutOptionsWidget){
            expect(factoredLayoutOptionsWidget).toBe(layoutOptionsWidget);

            zhLayoutCreateLayoutOptionsController.getCreateLayoutOptionsWidget(preference, configWidget).then(function(factoredLayoutOptionsWidgetSecondCall){
                expect(screenService.i18n).toHaveBeenCalledWith('Layout');
                expect(factoredLayoutOptionsWidgetSecondCall).toBe(factoredLayoutOptionsWidget);
                expect(zhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget.calls.count()).toEqual(1);
                expect(layoutOptionsWidget.fields).toEqual([
                    {'id': 1},
                    {'id': 2, 'name': 'field 2', 'label': 'Field 2', 'isVisible': true, 'sequenceInList': 0}
                ]);
                done();
            });
        });
    });

    it('saveAndApplyLayout', function(done){
        var listWidget = {
            'id': 'listWidget',
            'activate': jasmine.createSpy('activate')
        };

        var configWidget = {
            'id': 'configWidget',
            'listWidget': listWidget,
            'dataSource': {'name': 'dataSource'}
        };

        var preference = {
            'LAYOUT_CHANGES': '{"widgets": [{"id": "listWidget", "fields": []}]}'
        };

        var createLayoutOptionsWidget = {
            'gridConfigWidget': {
                'isValid': function(){ return true; }
            },
            'createNewLayoutWidget': {
                'configWidget': configWidget
            },
            'fields': [
                {
                    'id': 1,
                    'name': 'field 1',
                    'label': 'Field 1',
                    'isVisible': true,
                },
                {
                    'id': 2,
                    'name': 'field 2',
                    'label': 'Field 2',
                    'isVisible': true,
                    'sequenceInList': 0
                }
            ],
            'dataSource': {
                'data': []
            },
            'layoutName': 'Layout Name',
            'preference': preference,
            'isValid': function(){ return true; }
        };

        zhLayoutPreferences.saveLayouts.and.callFake(function(){
            return Q.when([preference]);
        });

        zhLayoutPreferences.applyLayout.and.callFake(function(){
            return Q.when();
        });

        zhLayoutCreateLayoutOptionsController.saveAndApplyLayout(createLayoutOptionsWidget).then(function(){
            expect(zhLayoutPreferences.saveLayouts).toHaveBeenCalledWith([preference]);
            expect(zhLayoutPreferences.applyLayout).toHaveBeenCalledWith(listWidget, preference);
            expect(screenService.closePopup).toHaveBeenCalledWith(true);
            expect(zhLayoutPreferences.updateLayouts).toHaveBeenCalledWith(listWidget.id, configWidget.dataSource);
            expect(listWidget.activate).toHaveBeenCalled();
            done();
        });
    });

    it('saveAndApplyLayout: without label', function(done){
        var createLayoutOptionsWidget = {
            'gridConfigWidget': {
                'isValid': function(){
                    return false;
                }
            }
        };
        zhLayoutCreateLayoutOptionsController.saveAndApplyLayout(createLayoutOptionsWidget).catch(function(err){
            expect(err).toBe('Invalid params');
            done();
        });
    });

    it('saveAndApplyLayout: with error', function(done){
        var listWidget = {
            'id': 'listWidget',
            'activate': jasmine.createSpy('activate')
        };

        var configWidget = {
            'id': 'configWidget',
            'listWidget': listWidget,
            'dataSource': {'name': 'dataSource'}
        };

        var preference = {
            'LAYOUT_CHANGES': '{"widgets": [{"id": "listWidget", "fields": []}]}'
        };

        var createLayoutOptionsWidget = {
            'gridConfigWidget': {
                'isValid': function(){ return true; }
            },
            'createNewLayoutWidget': {
                'configWidget': configWidget
            },
            'fields': [
                {
                    'id': 1,
                    'name': 'field 1',
                    'label': 'Field 1',
                    'isVisible': true,
                },
                {
                    'id': 2,
                    'name': 'field 2',
                    'label': 'Field 2',
                    'isVisible': true,
                    'sequenceInList': 0
                }
            ],
            'dataSource': {
                'data': []
            },
            'layoutName': 'Layout Name',
            'preference': preference,
            'isValid': function(){ return true; }
        };

        zhLayoutPreferences.saveLayouts.and.callFake(function(){
            return Q.reject(new Error('saveAndApplyLayout error'));
        });

        zhLayoutCreateLayoutOptionsController.saveAndApplyLayout(createLayoutOptionsWidget).then(function(){
            expect(zhLayoutPreferences.saveLayouts).toHaveBeenCalledWith([preference]);
            expect(zhLayoutPreferences.applyLayout).not.toHaveBeenCalled();
            expect(screenService.closePopup).not.toHaveBeenCalled();
            expect(zhLayoutPreferences.updateLayouts).not.toHaveBeenCalled();
            expect(listWidget.activate).not.toHaveBeenCalled();
            expect(messageService.showMessageByCode).toHaveBeenCalledWith('zh-layout-preferences#save_error', {'ERROR': 'saveAndApplyLayout error'});
            done();
        });
    });

    it('updateFieldVisibility', function(){
        var field = {'visibility': true};

        zhLayoutCreateLayoutOptionsController.updateFieldVisibility(field);

        expect(field).toEqual({
            isVisible: true,
            showOnList: true,
            isVisibleOnReport: true,
            visibility: true
        });
    });


});